<?php

namespace Teknisa\SUP\Controllers;

use Teknisa\SUP\Controllers\ReportWorkflowController as Report;
use Teknisa\SUP\Services\AprovacaoAlcadaService as Service;
use Teknisa\Sup\Controllers\SupplyTekApprovalWorkflowController as SupplyTekApproval;
use Zeedhi\Framework\DTO\Request\Row;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DTO\Response\Error;

class AprovacaoAlcadasController
{
    private $totalAFAtual = 0;
    private $valorDemanda = 0;
    private $valorAprovacaoAtual;
    private $supplyTekApproval;
    /*
    // Descontinuado
    private $totalPeriodo;
    private $totalOrcamento;
    */
    
    public function __construct(Service $service, Report $report, SupplyTekApproval $supplyTekApproval)
    {
        $this->service = $service;
        $this->report = $report;
        $this->supplyTekApproval = $supplyTekApproval;
    }

    public function getRequestToAprrove(Row $request, Response $response)
    {
        try {
            $row = $request->getRow();
            $idSisDos = $this->service->getIdSisDos();
            $order = $this->service->getApproverOrder($row);
            
            if (isset($order[0]) && $order[0]["IDEXIGETAPROVE"] == "S" || $idSisDos[0]["IDSISDOS"] == "3") {
                $rowToSend = self::arrayWorkflow($row);
                $this->supplyTekApproval->startWorkflow($rowToSend, $response);
            } else {
                if ($row['OPERACAO'] == 'APROVAR') {
                    self::orderApprove($row);
                } else if ($row['OPERACAO'] == 'REPROVAR') {
                    self::orderRepprove($row);
                }
            }
            if ($row['OPERACAO'] == 'APROVAR') {
                $messageToResponse = "Aprovada com sucesso!";
            } else if ($row['OPERACAO'] == 'REPROVAR') {
                $messageToResponse = "Reprovada com sucesso!";
            }
            $messageToResponse = new Response\Message($messageToResponse);
            $response->addMessage($messageToResponse);
            
            
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    // Adaptado de 'General.js'
    private function valueToFloat($value) {
        if (empty($value)) return 0.0;
        
		if (is_string($value) && $value) {
		    
		    $value = str_replace("R$", "", $value);
		    
			if (strpos($value, ',') === false && strpos($value, '.') === false) {
				return floatval($value);
			} else if (strpos($value, ',') > strpos($value, '.')) {
                $value = str_replace('.', '', $value);
                $value = str_replace(',', '.', $value);
				return floatval($value);
			} else if (strpos($value, ',') < strpos($value, '.')) {
			    $value = str_replace(',', '.', $value);
				return floatval($value);
			} else if (strpos($value, ',') === 0 || strpos($value, '.') === 0) {
			    if (strpos($value, ',') === 0) {
			        $value = str_replace(',', '.', $value);
			    }
				return floatval($value);    
			}
		} else {
			return $value;
		}
    }

    public function orderApprove($approvalData)
    {
        $orderTag = 'APROVADO';
        $orderTagFinalize = 'APROVADA';
        $justifyMessage = $approvalData['JUSTIFICATIVA'] == '' ? 'Aprovado.' : $approvalData['JUSTIFICATIVA'];
        $orderData = $this->service->getOrder($approvalData);
        $maxValue = $this->service->getApproverMaxValue($orderData)['VALORMAXIMO'];
        
        $paramValorUtilizado = self::getParamConsisteValorAprovacao($orderData);
        $totalOrderValue = self::getTotal($orderData, $paramValorUtilizado, $approvalData);

        if ($totalOrderValue > $maxValue) {
            if ($paramValorUtilizado == 'M') {
                throw new \Exception('Não existe gestor com valor de aprovação maior que o valor total da AF no período.');
            }
            
            throw new \Exception('Não existe gestor com valor de aprovação maior que o valor total da AF.');
        }
        
        // DEBUG - Comentar a linha abaixo para que não seja realizado o update.
        self::updateApprovalWorkflow($approvalData, $orderData, $justifyMessage, $orderTag);
        $subjectParam = self::setAprroveData($orderData, $justifyMessage, $orderTagFinalize, $approvalData, $totalOrderValue);
        $this->report->sendApprovalReport($orderData, $subjectParam, $justifyMessage, $approvalData['CDOPERLOG']);
    }

    public function orderRepprove($approvalData)
    {
        $orderTagUpdate = 'RECUSADO';
        $orderTagFinalize = 'RECUSADA';
        $subjectParam = 'REPROVACAO_ALCADA';
        $justifyMessage = $approvalData['JUSTIFICATIVA'];
        $orderData = $this->service->getOrder($approvalData);
        if (!empty($justifyMessage)) {
            self::updateApprovalWorkflow($approvalData, $orderData, $justifyMessage, $orderTagUpdate);
            self::finalizeWorkflow($orderData, $orderTagFinalize);
            $this->report->sendRepproveReport($orderData, $subjectParam, $justifyMessage, $approvalData['CDOPERLOG']);
        }
    }

    private function updateApprovalWorkflow($approvalData, $orderData, $justifyMessage, $orderTag)
    {
        $userLog = $approvalData['CDOPERLOG'];
        $approveCode = $orderData['NRAPROVALCADA'];
        $this->service->updateWorkflow($approveCode, $orderTag, $justifyMessage, $userLog);
    }

    private function finalizeWorkflow($orderData, $orderTag)
    {
        $nrAutoForn = $orderData['NRAUTOFORN'];
        $cdSupplier = $orderData['CDFORNECED'];
        $this->service->updateOrder($orderData, $orderTag);
        $this->service->executeCONSISTE_ORCAMENTO($orderData['CDFILDEST'], $orderData['CDFILLANC'], $orderData['NRAUTOFORN']);
    }

    private function setApprovalOnWorkflow($orderData, $userCode, $justifyMessage)
    {
        $nrApprove = $orderData['NRAPROVALCADA'];
        $this->service->setApprover($userCode, $nrApprove);
    }

    private function setAprroveData($orderData, $justifyMessage, $orderTag, $approvalData, $totalOrderValue)
    {
        $approver = $this->service->getApprover($orderData);
        
        if (empty($approver)) {
            self::finalizeWorkflow($orderData, $orderTag);
            
            return 'LIBERACAO_DEMANDA';
        }
        
        /*
        // DEBUG
        $excedeu = $this->isValorExcedente() ? 'Sim' : 'Não';
        echo("========================================\n");
        echo(">> function setAprroveData << \n");
        echo('Valor total excedeu? ');
        var_dump($excedeu);
        die;
        */
        
        if ($this->isValorExcedente()) {
            $userCode = $approver[0]['CDOPERADOR'];
            self::setApprovalOnWorkflow($orderData, $userCode, $justifyMessage);
            
            return 'APROVACAO_ALCADA';
        }
        
        self::finalizeWorkflow($orderData, $orderTag);
        
        return 'APROVACAO_AUTOMATICA';
    }
    
    /**
     * Verifica se o total disponível ultrapassa o permitido, sendo necessário
     * ir para o próximo aprovador.
     * 
     * Se o valor for positivo, significa que o valor ultrapassou o total disponível e,
     * portanto, deve ir para o próximo aprovador.
     * Do contrário, o valor está dentro do permitido e é aprovado automaticamente.
     */
    private function isValorExcedente()
    {
        $totalDisponivel = $this->getTotalDisponivel();
        
        /*
        // DEBUG
        echo("========================================\n");
        echo(">> function isValorExcedente << \n");
        echo('Total Disponível: ');
        var_dump($totalDisponivel);
        */
        
        return $totalDisponivel > 0;
    }
    
    /**
     * Retorna o total disponível, calculado entre Total Gasto menos o Total Disponível para Uso.
     * 
     * Os valores que compõe os valores utilizados no cálculo são recuperados
     * na função getTotalGastoNoPeriodo.
     */
    private function getTotalDisponivel()
    {
        return $this->getTotalGastoNoPeriodo() - $this->getTotalDisponivelParaUso();
    }
    
    /**
     * O total gasto é a soma dos seguintes valores:
     * - resultado da função VALOR_DEMANDA (detalhes do funcionamento no service);
     * - valor total da AF atual.
     */
    private function getTotalGastoNoPeriodo()
    {
        /*
        // DEBUG
        echo("========================================\n");
        echo(">> function getTotalGasto << \n");
        echo('Total Demanda: ');
        var_dump($this->valorDemanda);
        echo('Total AF Atual: ');
        var_dump($this->totalAFAtual);
        echo('Soma: ');
        var_dump($this->valorDemanda + $this->totalAFAtual);
        */
        
        // return $this->totalPeriodo + $this->totalAFAtual;
        return $this->valorDemanda + $this->totalAFAtual;
    }
    
    /**
     * Total disponível para uso é o valor de aprovação do aprovador atual.
     * O valor do orçamento disponível já foi descontado na VALOR_DEMANDA.
     */
    private function getTotalDisponivelParaUso()
    {
        /*
        // DEBUG
        echo("========================================\n");
        echo(">> function getTotalDisponivelParaUso << \n");
        echo('Valor de Aprovação do Aprovador Atual: ');
        var_dump($this->valorAprovacaoAtual);
        */
        
        // return $this->totalOrcamento + $this->valorAprovacaoAtual;
        return $this->valorAprovacaoAtual;
    }
    
    public function transformValueMaskToFloat($value)
    {
        $arrayReplace = array(
            'R$',
            '.',
        );
        
        $newValue = str_replace(',', '.', str_replace($arrayReplace, '', $value));
        
        return (float) $newValue;
    }

    public function getTotal($dadosDemanda, $paramValorUtilizado, $approvalData)
    {
        if ($paramValorUtilizado == 'M') {
            $dataEntregaAF    = $this->getDataEntregaAF($dadosDemanda);
            $periodoOrcamento = $this->getPeriodoOrcamento($dadosDemanda, $dataEntregaAF);
            $aprovadorAtual = $this->service->findAprovadorAtual($approvalData)[0];
            
            $this->valorAprovacaoAtual = $aprovadorAtual['VRAPROVCMP'];
            
            $this->valorDemanda = $this->getValorDemanda($dadosDemanda, $dataEntregaAF, $periodoOrcamento);
            $this->totalAFAtual = $dadosDemanda['VRTOTALAF'];
            
            $totalGasto = $this->getTotalGastoNoPeriodo();
            
            /*
            // Descontinuado
            $produtoOrcamento = $this->getProdutoOrcamento($dadosDemanda, $periodoOrcamento, $dataEntregaAF)[0];
            $this->totalOrcamento = $produtoOrcamento['VRORCAMENTO'];
            $this->totalPeriodo = self::getTotalPeriodo($dadosDemanda, $dataEntregaAF, $produtoOrcamento);
            $totalGasto = $this->totalPeriodo + $dadosDemanda['VRTOTALAF'];
            */
            /*
            // OLD DEBUG
            echo("========================================\n");
            echo(">> function getTotal << \n");
            echo('Codigo do Produto: ');
            var_dump($produtoOrcamento['CDARVPROD']);
            echo('Valor do Orçamento: ');
            var_dump($produtoOrcamento['VRORCAMENTO']);
            */
        } else {
            $totalGasto = self::getTotalDia($dadosDemanda) + $dadosDemanda['VRTOTALAF'];
        }
        
        return $totalGasto;
    }
    
    /*
    // Descontinuado
    public function getTotalPeriodo($dadosDemanda, $dataEntregaAF, $produtoOrcamento)
    {
        if ($dadosDemanda['IDTPDEMANDA'] == 'AF') {
            $paramsTotalPeriodo = [
                'DTPROGENTAF' => $dataEntregaAF,
                'CDFILDEST'   => $dadosDemanda['CDFILDEST'],
                'NRDEMANDA'   => $dadosDemanda['NRAUTOFORN'],
                'CDARVPROD'   => $produtoOrcamento['CDARVPROD'],
            ];
            
            return $this->service->getTotalPeriodoAF($paramsTotalPeriodo)[0]['VR_TOTAL_AF'];
        }
    }*/
    
    public function getValorDemanda($dadosDemanda, $dataEntregaAF, $periodoOrcamento)
    {
        $paramsValorDemanda = [
            'DTPROGENTAF' => $dataEntregaAF,
            'CDFILDEST'   => $dadosDemanda['CDFILDEST'],
            'NRDEMANDA'   => $dadosDemanda['NRAUTOFORN'],
            'IDPERIODOORCA'   => $periodoOrcamento['V_IDPERIODOORCA'],
            'NRPERIODOORCA'   => $periodoOrcamento['V_NRPERIODOORCAGOP'],
        ];
        
        return $this->service->getValorDemanda($paramsValorDemanda)[0]['VRDEMANDA'];
    }
    
    public function getTotalDia($dadosDemanda)
    {
        $params = [
            "CDFILDEST"    => $dadosDemanda['CDFILDEST'],
            "CDFILLANC"    => $dadosDemanda['CDFILLANC'],
            "NRDEMANDA"    => $dadosDemanda['NRAUTOFORN'],
        ];
        
        return $this->service->totalDiaAF($params)[0]['TOTAL'];
    }
    
    /*
    // Descontinuado
    public function getProdutoOrcamento($dadosDemanda, $periodoOrcamento, $dataEntregaAF)
    {
        $paramsfindProdutoOrcamento = [
            'IDPERIODOORCA' => $periodoOrcamento['V_IDPERIODOORCA'],
            'NRPERIODOORCA' => $periodoOrcamento['V_NRPERIODOORCAGOP'],
            'DTPROGENTAF'   => $dataEntregaAF,
            'CDFILDEST'     => $dadosDemanda['CDFILDEST'],
            'CDFILLANC'     => $dadosDemanda['CDFILLANC'],
            'NRDEMANDA'     => $dadosDemanda['NRAUTOFORN'],
        ];
            
        return $this->service->findProdutoOrcamento($paramsfindProdutoOrcamento);
    }*/
    
    public function getDiaInicioOrcamento($dadosDemanda)
    {
        return $this->service->findDiaInicioOrcamento($dadosDemanda['CDFILDEST'])[0]['DIAINIORCA'];
    }
    
    public function getPeriodoOrcamento($dadosDemanda, $dataEntregaAF)
    {
        $diaInicioOrcamento = $this->getDiaInicioOrcamento($dadosDemanda);
        
        $dataOrcamento = $this->getDataOrcamento($diaInicioOrcamento, $dataEntregaAF);
        
        $paramsPeriodoOrcamento = [
            'V_DATAINI' => $dataOrcamento['V_DATAINI'],
            'DIAINIORCA' => $diaInicioOrcamento,
        ];
        
        return $this->service->findPeriodoOrcamento($paramsPeriodoOrcamento)[0];
    }
    
    public function getDataOrcamento($diaInicioOrcamento, $dataEntregaAF)
    {
        $params = [
            'DIA' => $diaInicioOrcamento,
            'DATA' => $dataEntregaAF,
        ];
        
        return $this->service->findDataOrcamento($params)[0];
    }
    
    public function getDataEntregaAF($dadosDemanda)
    {
        $params = [
            'CDFILDEST' => $dadosDemanda['CDFILDEST'],
            'CDFILLANC' => $dadosDemanda['CDFILLANC'],
            'NRDEMANDA' => $dadosDemanda['NRAUTOFORN'],
        ];
        
        return $this->service->findDataEntregaAF($params)[0]['DTPROGENTAF'];
    }
    
    public function getParamConsisteValorAprovacao($dadosDemanda)
    {
        $params = [
            'CDFILDEST'      => $dadosDemanda['CDFILDEST'],
            'NRCONFIGALCADA' => $dadosDemanda['NRCONFIGALCADA'],
        ];
        
        return $this->service->findFilialAlcada($params)[0]['IDCONSISTVRAPROV'];
    }
    
    //////////////////////////////////////////////////////////////////////////////////////////////////
    ///                              FUNÇÔES EM DESENVOLVIMENTO                                   ////
    //////////////////////////////////////////////////////////////////////////////////////////////////
    public function arrayWorkflow($row)
    {
        $vrAprovCmp = $this->valueToFloat($row["VRAPROVACAO"]);
        $row["JUSTIFICATIVA"] = $row['JUSTIFICATIVA'] == '' ? 'Aprovado.' : $row['JUSTIFICATIVA'];
        
        $rowToSend = array(
            "CDFILDEST"         => $row["CDFILDEST"],
            "CDFILLANC"         => $row["CDFILLANC"],
            "NRAUTOFORN"        => $row["NRAUTOFORNE"],
            "CDOPERADOR"        => $row["CDOPERLOG"],
            "STATUS"            => $row["OPERACAO"][0],
            "DSJUSTIFIC"        => $row["JUSTIFICATIVA"],
            "NRAPROVALCADA"     => $row["NRAPROVALCADA"],
            "NRCONFIGALCADA"    => $row["NRCONFIGALCADA"],
            "VRAPROVCMP"        => $vrAprovCmp
        );
        
        return $rowToSend;
    }
}
