<?php
namespace Teknisa\SUP\Modules\Relatorio\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;

class Sup02100EvolucaoComprasController extends \Zeedhi\Framework\Controller\Report
{
    protected $entityManager;
    protected $birtStrategy;
    protected $newCode;
    protected $environment;
    protected $connection;
    protected $dataSourceManager;
    
    public function __construct($reportService, $entityManager, $birtStrategy, $newCode, $environment, $connection, $dataSourceManager)
    {
        $this->entityManager     = $entityManager;
        $this->birtStrategy      = $birtStrategy;
        $this->reportService     = $reportService;
        $this->newCode           = $newCode;
        $this->environment       = $environment;
        $this->connection        = $connection;
        $this->dataSourceManager = $dataSourceManager;
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response)
    {
        $row = $request->getRow();
        
        $row['DTINICIAL'] = $row['DTINI'];
        $row['DTFINAL'] = $row['DTFIN'];

        $row['CDARVPRODFIN'] = str_pad($row['CDARVPRODFIN'], 10, 'Z', STR_PAD_RIGHT);
        $row['NRORG'] = $this->environment->getNrOrg();
        
        
        //criação da lista para Unidade, Comprador e Fornecedor
        if($row['STRATEGY'] == 'BIRT'){
            if($row['CDFILIAL'] != 'T'){
                $nrlistaForn = $this->criarListaValores($row['CDFILIAL']);   
                $row['CDFILIAL'] = "_NRLISTA=".$nrlistaForn;
            }
            if($row['CDFORNECED'] != 'T'){
                 $nrlistaForn= $this->criarListaValores($row['CDFORNECED']);
                 $row['CDFORNECED'] = "_NRLISTA=".$nrlistaForn;
            }
            if($row['CDOPERADOR'] != 'T'){
                $nrlistaForn = $this->criarListaValores($row['CDOPERADOR']);
                $row['CDOPERADOR'] = "_NRLISTA=".$nrlistaForn;
            }

        }
        $row['CDFORNECED1'] = $row['CDFORNECED'];
        $row['CDFILIAL1'] = $row['CDFILIAL'];
        $row['CDOPERADOR1'] = $row['CDOPERADOR'];
        
        // print_r($row).die();
   
        $this->birtStrategy->setReportFormat($row['QR2TYPE']);
        $this->stNomeRel = $this->getReportName($row);
        $this->reportMapping[$this->stNomeRel] =
            array (
                'strategy' => $row['STRATEGY'],
                'parameterMapping' =>
                    array (
                        "CDFILIAL"      => "L_CDFILDEST",
                        "CDOPERADOR"    => "L_CDOPERADOR",
                        "CDFORNECED"    => "L_CDFORNECED",
                        "CDFILIAL1"      =>"L_CDFILDEST1",
                        "CDOPERADOR1"    =>"L_CDOPERADOR1",
                        "CDFORNECED1"    =>"L_CDFORNECED1",
                        "DTINICIAL"     => "P_DTINICIAL",
                        "DTFINAL"       => "P_DTFINAL",
                        "CDARVPRODINI"  => "P_CDPRODINIC",
                        "CDARVPRODFIN"  => "P_CDPRODFIN",
                        "DTINI"         => "P_CDDATINIC",
                        "DTFIN"         => "P_CDDATFIN",
                        "NRORG"         => "P_NRORG"
                    ),
            );
    }
    
    //cria lista de valores recebendo string como parametro
    protected function criarListaValores($lista){
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrg());
        $valores = explode('_',$lista);
        try{
            foreach($valores as $key){
                $this->insertLista($nrlista, $key);
            }
            return $nrlista;
        } catch(\Exception $e){
            throw $e;
        }
     }
    
    //insere lista no banco
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
}
